==> maps_suite-7.x-2.0.tar.gz: maps_suite/modules/import/lib/Drupal/maps_import/Mapping/Mapping.php <==
<?php

/**
 * @file
 * @todo make this exportable using ctools module.
 * Abstract class for Mapping.
 */

namespace Drupal\maps_import\Mapping;

use Drupal\maps_import\Cache\MapsSystemPropertyWrapper;
use Drupal\maps_import\Converter\ConverterInterface;
use Drupal\maps_import\Mapping\Mapper\Mapper;
use Drupal\maps_import\Mapping\Item\Item;
use Drupal\maps_import\Mapping\Source\MapsSystem as MS;
use Drupal\maps_import\Mapping\Source\MapsSystem\Attribute\Broken as BrokenAttribute;
use Drupal\maps_import\Mapping\Target\Drupal as Drupal;
use Drupal\maps_import\Mapping\Target\Drupal\Field\DefaultField;
use Drupal\maps_import\Exception\MappingException;
use Drupal\maps_import\Mapping\Target\Drupal\EntityInterface as DrupalEntityInterface;
use Drupal\maps\Log\Context\Context as LogContext;

/**
 * Base MaPS Import Mapping class.
 */
abstract class Mapping implements MappingInterface {

  /**
   * Stock the current MaPS System entity.
   *
   * @var MS\Entity
   */
  public static $currentEntity;

  /**
   * The related converter.
   *
   * @var ConverterInterface
   */
  private $converter;

  /**
   * The array of mapping items.
   *
   * @var ArrayObject
   */
  protected $items;

  /**
   * The mapping type.
   *
   * @var string
   */
  protected $type;

  /**
   * @inheritdoc
   */
  public function __construct(ConverterInterface $converter) {
    $this->converter = $converter;
  }

  /**
   * @inheritdoc
   */
  public function getConverter() {
    return $this->converter;
  }

  /**
   * @inheritdoc
   */
  public function getType() {
    return $this->type;
  }

  /**
   * @inheritdoc
   */
  public function process(MS\Entity $entity) {
    self::$currentEntity = $entity;

    Mapper::log()
      ->addContext(new LogContext('source'), 'child')
      ->addContext(new LogContext('properties'), 'child');

    try {
      // Set raw values in MaPS Entity.
      foreach ($this->getItems() as $item) {
        $entity->setRawValues($item->getProperty(), $item->getOptions());
      }

      // Retrieve existing entities.
      $uidProperty = MS\PropertyWrapper::load($this->getConverter(), $this->getConverter()->getUid());
      $uid = $uidProperty->getValues($entity);
      $uid = reset($uid);

      $existingEntities = array();
      $existingEntities = $this->getMappedEntities($uid);

      // Retrieve entity information and Drupal Entity class.
      $entityInfo = entity_get_info($this->getConverter()->getEntityType());
      $drupalEntityClass = (class_exists($entityInfo['maps_import_handler'])) ? $entityInfo['maps_import_handler'] : 'Drupal\\maps_import\\Mapping\\Target\\Drupal\\Entity';

      Mapper::log()
        ->moveToParent('source')
        ->moveToParent()
        ->addContext(new LogContext('target'), 'child')
        ->addContext(new LogContext('fields'), 'child');

      // Create a new Drupal Entity.
      $drupalEntity = new $drupalEntityClass($this->getConverter(), $existingEntities);

      // Set values in Drupal Entity.
      foreach ($this->getItems() as $item) {
        $drupalEntity->getTranslationHandler()->setValue($item->getField(), $item->getProperty(), $entity, $item->isRequired());
      }

      try {
	      // Save the Drupal Entity.
	      $drupalEntity->presave($this->getConverter()->getEntityType(), $this->getConverter()->getBundle());
	      $drupalEntity->save();

	      // Insert the identifiers in maps_import tables.
	      $identifiers = $drupalEntity->getIdentifiers();
	      foreach ($identifiers as $id_language => $identifier) {
          $op = 'update';

	      	$fields = array(
            'entity_type' => $this->getConverter()->getEntityType(),
            'bundle' => $this->getConverter()->getBundle(),
            'uid' => $uid,
            'uid_scope' => $this->getConverter()->getUidScope(),
            'entity_id' => $identifier,
            'id_language' => (!empty($id_language)) ? $id_language : 0,
            'pid' => $this->getConverter()->getProfile()->getPid(),
            'cid' => $this->getConverter()->getCid(),
          );

	      	// We check if there is already these values in DB_ENTITIES_TABLE;
	      	$query_select = db_select(MappingInterface::DB_ENTITIES_TABLE)
	      	  ->fields(MappingInterface::DB_ENTITIES_TABLE, array('id'));
	      	foreach ($fields as $index => $value) {
	      		$query_select->condition($index, $value);
	      	}
	      	$id = $query_select->execute()->fetchField();

	      	if (!$id) {
	          $id = db_insert(MappingInterface::DB_ENTITIES_TABLE)
	            ->fields($fields)
	            ->execute();
		      	$op = 'insert';
	      	}

	      	$pk = $op == 'update' ? array('maps_id', 'correspondence_id') : array();
          $record = array(
            'maps_id' => $entity->getId(),
            'correspondence_id' => $id,
            'updated' => time(),
          );

          drupal_write_record('maps_import_' . str_replace('_mapping', '', $this->getType()) . '_ids', $record, $pk);

	      	// Manage delayed entities.
	        foreach ($this->getItems(TRUE) as $item) {
	          $options = $item->getOptions();
	          foreach ($entity->getRelatedEntities() as $relatedEntity) {
	          	// Retrieve the parent id.
	            $parentId = $relatedEntity[$item->getProperty()->getId()];
              $parentEntityIds = self::getEntityIdFromMapsId($this->getConverter()->getType(), $parentId, $this->getConverter()->getUidScope());

	            if (!$parentEntityIds || !$parentEntities = entity_load($options['entity_type'], $parentEntityIds)) {
	              continue;
	            }

	            // Get the values.
	            $values = array();
	            foreach ($parentEntities as $parentEntity) {
	              $wrapper = entity_metadata_wrapper($options['entity_type'], $parentEntity);
	              $values[] = $item->getField()->sanitize($identifier);

	              // Check if the entity has the given field.
	              if (!isset($wrapper->{$item->getField()->getId()})) {
	                continue;
	              }

	              // Replace old datas
	              if ($options['update_mode'] == 'replace') {
	                $wrapperValues = $wrapper->value();
	                foreach (reset($wrapperValues->{$item->getField()->getId()}) as $wrapperValue) {
	                  $value = reset($wrapperValue);
	                  if (empty($values[$value])) {
	                    $values[$value] = $value;
	                  }
	                }
	                $wrapper->{$item->getField()->getId()}->set(NULL);
	              }

	              // Set the values in the wrapper.
	              foreach ($values as $value) {
	                $wrapper->{$item->getField()->getId()}[$value]->set($value);
	              }
	            }

	            try {
  	            $wrapper->save();
	            }
	            catch(\Exception $e)
	            {
	            	Mapper::log()->addMessageFromException($e);
	            }
	          }
	        }
	      }

	      // Check if the entity is published.
	      if (!$entity->isPublished()) {
	        $drupalEntity->unpublishEntities($this->getConverter());
	      }
      }
      catch (\EntityMalformedException $e) {
	      Mapper::log()
	        ->addContext(new LogContext('error'), 'child')
	        ->addMessageFromException($e);
      }
    }
    catch (MappingException $e) {
      Mapper::log()
        ->addContext(new LogContext('error'), 'child')
        ->addMessageFromException($e);

      return array();
    }

    return array($entity, $drupalEntity);
  }

  /**
   * @inheritdoc
   */
  public function getItems($getDelayedEntity = FALSE) {
    $items = array();

    $type = $this->getConverter()->getType() == 'link' ? 'link' : 'object';

    $result = db_select('maps_import_mapping_item')
      ->fields('maps_import_mapping_item')
      ->condition('cid', $this->getConverter()->getCid())
      ->condition('type', $type)
      ->orderBy('weight')
      ->execute()
      ->fetchAll();

    foreach ($result as $item) {
      $options = unserialize($item->options);

      if ($getDelayedEntity && (!isset($options['post_save']) || !$options['post_save'])) {
        continue;
      }

      if (!$getDelayedEntity && isset($options['post_save']) && $options['post_save']) {
        continue;
      }

      $converter = $this->converter;
      if ($getDelayedEntity) {
        $converterClass = "Drupal\\maps_import\\Converter\\" . maps_drupal2camelcase($this->converter->getType());
        $converter = new $converterClass($this->converter->getProfile());
        $converter->setBundle($options['bundle']);
        $converter->setEntityType($options['entity_type']);
      }

      $property = MS\PropertyWrapper::load($converter, $item->property_id);
      $field = DefaultField::load($converter, $item->field_name);

      $items[$item->id] = new Item(array(
        'id' => $item->id,
        'property' => $property,
        'field' => $field,
        'weight' => $item->weight,
        'required' => $item->required,
        'options' => unserialize($item->options),
        'converter' => $converter,
      ));
    }

    return $items;
  }

  /**
   * @inheritdoc
   */
  public function getItem($id) {
    $items = $this->getItems();
    return isset($items[$id]) ? $items[$id] : FALSE;
  }

  /**
   * @inheritdoc
   */
  public function getSourceAttributes() {
    // @todo use a cache class.
    // @todo handle languages properly
    static $attributes;

    if (!$languageId = $this->getConverter()->getProfile()->getLanguage()) {
      return array();
    }

    if (!isset($attributes[$languageId])) {
      $attributes[$languageId] = array();

      foreach ($this->converter->getProfile()->getConfigurationTypes('attribute', $languageId) as $id => $attribute) {
        $attribute += unserialize($attribute['data']);
        unset($attribute['data']);

        if (isset($attribute['attribute_type_code'])) {
          $class = __NAMESPACE__ . '\\Source\\MapsSystem\\Attribute\\' . maps_drupal2camelcase($attribute['attribute_type_code']);
          $attributes[$languageId]['attribute:' . $id] = class_exists($class) ? new $class($attribute) : new MS\Attribute\DefaultAttribute($attribute);
        }
      }//end foreach
    }

    return $attributes[$languageId];
  }

  /**
   * @inheritdoc
   */
  public function getSourceAttribute($id) {
    $attributes = $this->getSourceAttributes();
    return isset($attributes[$id]) ? $attributes[$id] : new BrokenAttribute(array('id' => $id));
  }

  /**
   * Build the property handlers from property definitions.
   *
   * @param $properties
   *   The list of properties.
   *
   * @return array
   */
  protected function buildSourcePropertyHandlers(array $properties) {
    $handlers = array();

    foreach ($properties as $id => $property) {
      $property += array('class' => 'DefaultProperty', 'id' => $id);
      $class = __NAMESPACE__ . '\\Source\\MapsSystem\\Property\\' . $property['class'];

      if (class_exists($class) && @is_subclass_of($class, __NAMESPACE__ . '\\Source\\MapsSystem\\PropertyWrapper')) {
        $handlers['property:' . $id] = new $class($property);
      }
    }

    return $handlers;
  }

  /**
   * @inheritdoc
   */
  public function getSourceProperty($name) {
    $properties = $this->getSourceProperties();
    return isset($properties[$name]) ? $properties[$name] : new MS\Property\Broken();
  }

  /**
   * @inheritdoc
   */
  public function getTargetFields() {
    global $language;
    // @todo use a cache class.
    // @todo handle languages properly
    $fields = array();

    $info = entity_get_property_info($this->getConverter()->getEntityType()) + array('properties' => array());
    $properties = $info['properties'];

    if (isset($info['bundles'][$this->getConverter()->getBundle()]['properties'])) {
      $properties += $info['bundles'][$this->getConverter()->getBundle()]['properties'];
    }

    foreach ($properties as $name => $property) {
      if (isset($property['maps_import_handler'])) {
        $property['id'] = $name;
        $fields[$name] = new $property['maps_import_handler']($property);
      }
    }

    return $fields;
  }

  /**
   * @inheritdoc
   */
  public function getTargetField($name) {
    $fields = $this->getTargetFields();

    // @todo create the Broken class for Drupal fields and add an additional
    // use statement on the top of current file with an alias of BrokenField.
    return isset($fields[$name]) ? $fields[$name] : new Drupal\Field\Broken(array());
  }

  /**
   * @inheritdoc
   */
  public function getMappedEntities($uid) {
    $query = db_select(MappingInterface::DB_ENTITIES_TABLE, 'e')
      ->fields('e')
      ->condition('e.entity_type', $this->getConverter()->getEntityType())
      ->condition('e.bundle', $this->getConverter()->getBundle())
      ->condition('e.uid', $uid)
      ->condition(db_or()
        ->condition('e.uid_scope', 1)
        ->condition(db_and()
          ->condition('e.uid_scope', 2)
          ->condition('e.pid', $this->getConverter()->getProfile()->getPid())))
      ->orderBy('abs(e.pid - ' . $this->getConverter()->getProfile()->getPid() . ')', 'ASC')
      ->orderBy('abs(e.cid - ' . $this->getConverter()->getCid() . ')', 'ASC');

    // @todo this could override some other existing contents...
    $result = $query->execute()->fetchAllAssoc('id_language', \PDO::FETCH_ASSOC);
    return $result;
  }

  /**
   * @inheritdoc
   */
  public static function getEntityFromMapsId($type, $mapsId, $uid_scope = 1) {
  	$table = 'maps_import_' . $type . '_ids';

    $query = db_select(MappingInterface::DB_ENTITIES_TABLE, 'e');
    $query->join($table, 't', 'e.id = t.correspondence_id');
    $query
      ->fields('e')
      ->condition('e.uid_scope', $uid_scope)
      ->condition('t.maps_id', $mapsId);

    $result = $query->execute()->fetchAllAssoc('id_language', \PDO::FETCH_ASSOC);

    return !empty($result) ? $result : array();
  }

  /**
   * @inheritdoc
   */
  public static function getEntityIdFromMapsId($type, $mapsId, $uid_scope = 1) {
  	if ($result = self::getEntityFromMapsId($type, $mapsId, $uid_scope)) {
  		foreach ($result as &$row) {
        $row = $row['entity_id'];
  		}
    }

    return !empty($result) ? $result : array();
  }

  /**
   * @inheritdoc
   */
  public static function getCurrentEntity() {
    return self::$currentEntity;
  }

}

==> maps_suite-7.x-2.1.tar.gz: maps_suite/modules/import/lib/Drupal/maps_import/Mapping/Mapping.php <==
<?php

/**
 * @file
 * @todo make this exportable using ctools module.
 * Abstract class for Mapping.
 */

namespace Drupal\maps_import\Mapping;

use Drupal\maps_import\Cache\MapsSystemPropertyWrapper;
use Drupal\maps_import\Converter\ConverterInterface;
use Drupal\maps_import\Mapping\Mapper\Mapper;
use Drupal\maps_import\Relation\Relation;
use Drupal\maps_import\Mapping\Item\Item;
use Drupal\maps_import\Mapping\Source\MapsSystem as MS;
use Drupal\maps_import\Mapping\Source\MapsSystem\Attribute\Broken as BrokenAttribute;
use Drupal\maps_import\Mapping\Target\Drupal as Drupal;
use Drupal\maps_import\Mapping\Target\Drupal\Field\DefaultField;
use Drupal\maps_import\Exception\MappingException;
use Drupal\maps_import\Mapping\Target\Drupal\EntityInterface as DrupalEntityInterface;
use Drupal\maps_suite\Log\Context\Context as LogContext;
use Drupal\maps_import\Mapping\Target\Drupal\Relation as DrupalRelation;

/**
 * Base MaPS Import Mapping class.
 */
abstract class Mapping implements MappingInterface {

  /**
   * Stock the current MaPS System® entity.
   *
   * @var MS\Entity
   */
  public static $currentEntity;

  /**
   * The related converter.
   *
   * @var ConverterInterface
   */
  private $converter;

  /**
   * The array of mapping items.
   *
   * @var ArrayObject
   */
  protected $items;

  /**
   * The mapping type.
   *
   * @var string
   */
  protected $type;

  /**
   * @inheritdoc
   */
  public function __construct(ConverterInterface $converter) {
    $this->converter = $converter;
  }

  /**
   * @inheritdoc
   */
  public function getConverter() {
    return $this->converter;
  }

  /**
   * @inheritdoc
   */
  public function getType() {
    return $this->type;
  }

  /**
   * @inheritdoc
   */
  public function process(MS\Entity $entity) {
    self::$currentEntity = $entity;

    Mapper::log()
      ->addContext(new LogContext('source'), 'child')
      ->addContext(new LogContext('properties'), 'child');

    try {
      // Set raw values in MaPS Entity.
      foreach ($this->getItems() as $item) {
        $entity->setRawValues($item->getProperty(), $item->getOptions());
      }

      // Retrieve existing entities.
      $converterUid = $this->getConverter()->getParentId() > 0 ? $this->getConverter()->getParent()->getUid() : $this->getConverter()->getUid();
      $uidProperty = MS\PropertyWrapper::load($this->getConverter(), $converterUid);
      $uids = $uidProperty->getValues($entity);
      $uid = reset($uids);

      $existingEntities = $this->getMappedEntities($uid);

      // Retrieve entity information and Drupal Entity class.
      $entityInfo = entity_get_info($this->getConverter()->getEntityType());
      $drupalEntityClass = (class_exists($entityInfo['maps_import_handler'])) ? $entityInfo['maps_import_handler'] : 'Drupal\\maps_import\\Mapping\\Target\\Drupal\\Entity';

      Mapper::log()
        ->moveToParent('source')
        ->moveToParent()
        ->addContext(new LogContext('target'), 'child')
        ->addContext(new LogContext('fields'), 'child');

      // Create a new Drupal Entity.
      $drupalEntity = new $drupalEntityClass($this->getConverter(), $existingEntities);

      // Set values in Drupal Entity.
      foreach ($this->getItems() as $item) {
        if (!$drupalEntity->getTranslationHandler()->setValue($item->getField(), $item->getProperty(), $entity, $item->isRequired())) {
          return FALSE;
        }
      }

      try {
	      // Save the Drupal Entity.
	      $drupalEntity->presave($this->getConverter()->getEntityType(), $this->getConverter()->getBundle());
	      $drupalEntity->save();

	      // Insert the identifiers in maps_import tables.
	      $identifiers = $drupalEntity->getIdentifiers();
	      foreach ($identifiers as $id_language => $identifier) {
          $op = 'update';

	      	$fields = array(
            'entity_type' => $this->getConverter()->getEntityType(),
            'bundle' => $this->getConverter()->getBundle(),
            'uid' => $uid,
            'uid_scope' => $this->getConverter()->getUidScope(),
            'entity_id' => $identifier,
            'id_language' => (!empty($id_language)) ? $id_language : 0,
            'pid' => $this->getConverter()->getProfile()->getPid(),
            'cid' => $this->getConverter()->getCid(),
          );

	      	// We check if there is already these values in DB_ENTITIES_TABLE;
	      	$query_select = db_select(MappingInterface::DB_ENTITIES_TABLE)
	      	  ->fields(MappingInterface::DB_ENTITIES_TABLE, array('id'));
	      	foreach ($fields as $index => $value) {
	      		$query_select->condition($index, $value);
	      	}
	      	$id = $query_select->execute()->fetchField();

	      	if (!$id) {
	          $id = db_insert(MappingInterface::DB_ENTITIES_TABLE)
	            ->fields($fields)
	            ->execute();
		      	$op = 'insert';
	      	}

	      	$pk = $op == 'update' ? array('maps_id', 'correspondence_id') : array();
          $record = array(
            'maps_id' => $entity->getId(),
            'correspondence_id' => $id,
            'updated' => $entity->getUpdated(),
          );

          drupal_write_record('maps_import_' . str_replace('_mapping', '', $this->getType()) . '_ids', $record, $pk);

          // Post process.
          foreach ($this->getItems() as $item) {
            $item->getField()->postProcess($this->getConverter(), $entity, $drupalEntity);
          }

	      	// Manage delayed entities.
	        foreach ($this->getItems('delayed') as $item) {
	          $options = $item->getOptions();

	          foreach ($entity->getRelatedEntities() as $relatedEntity) {
	          	// Retrieve the parent id.
	            $parentId = $relatedEntity[$item->getProperty()->getId()];
              $parentEntityIds = self::getEntityIdFromMapsId($this->getConverter()->getType(), $parentId, $this->getConverter()->getUidScope());

	            if (!$parentEntityIds || !$parentEntities = entity_load($options['entity_type'], $parentEntityIds)) {
	              continue;
	            }

	            // Get the values.
	            $values = array();

	            foreach ($parentEntities as $parentEntity) {
	              $wrapper = entity_metadata_wrapper($options['entity_type'], $parentEntity);
	              $values[] = $item->getField()->sanitize($identifier);

	              // Check if the entity has the given field.
	              if (!isset($wrapper->{$item->getField()->getId()})) {
	                continue;
	              }

	              // Replace old datas
	              if ($options['update_mode'] == 'replace') {
	                $wrapperValues = $wrapper->value();
	                foreach (reset($wrapperValues->{$item->getField()->getId()}) as $wrapperValue) {
	                  $value = reset($wrapperValue);
	                  if (empty($values[$value])) {
	                    $values[$value] = $value;
	                  }
	                }
	                $wrapper->{$item->getField()->getId()}->set(NULL);
	              }

	              // Set the values in the wrapper.
	              foreach ($values as $value) {
	                $wrapper->{$item->getField()->getId()}[$value]->set($value);
	              }
	            }

	            try {
  	            $wrapper->save();
	            }
	            catch(\Exception $e) {
	            	Mapper::log()->addMessageFromException($e);
	            }
	          }
	        }
          
          // Manage relations
          if (module_exists('relation')) {
            $relations = Relation::loadAll($this->getConverter());
            
            foreach ($relations as $relation) {
              DrupalRelation::deleteExistingRelations($relation->getMappingConverter());
              $relation->getMappingConverter()->getMapping()->process($entity);
            }            
          }
	      }

        // Check if the entity is published.
	      if (!$entity->isPublished()) {
	        $drupalEntity->unpublishEntities($this->getConverter());
	      }
      }
      catch (\EntityMalformedException $e) {
	      Mapper::log()
	        ->addContext(new LogContext('error'), 'child')
	        ->addMessageFromException($e);
      }
    }
    catch (MappingException $e) {
      Mapper::log()
        ->addContext(new LogContext('error'), 'child')
        ->addMessageFromException($e);

      return array();
    }

    return array($entity, $drupalEntity);
  }

  /**
   * @inheritdoc
   *
   * @todo create a Cache class for this.
   */
  public function getItems($type = 'default') {
    $items = array();

    $result = db_select('maps_import_mapping_item')
      ->fields('maps_import_mapping_item')
      ->condition('cid', $this->getConverter()->getCid())
      // Object and Media converters are stored in the same table, but not the Link ones.
      ->condition('type', $this->getConverter()->getType() === 'link' ? 'link' : 'object')
      ->orderBy('weight')
      ->execute()
      ->fetchAll();

    foreach ($result as $item) {
      $options = unserialize($item->options);

      if ($type !== 'all' && !($type === 'delayed' xor empty($options['post_save']))) {
        continue;
      }

      $converter = $this->getConverter();
      if ($type === 'delayed') {
        $class = get_class($this->getConverter());
        $converter = new $class($this->getConverter()->getProfile());
        $converter->setBundle($options['bundle']);
        $converter->setEntityType($options['entity_type']);
      }

      $property = MS\PropertyWrapper::load($converter, $item->property_id);
      $field = DefaultField::load($converter, $item->field_name);
      
      if (!empty($options['target'])) {
        $field->setOptions($options['target']);
      }
      
      $items[$item->id] = new Item(array(
        'id' => $item->id,
        'property' => $property,
        'field' => $field,
        'weight' => $item->weight,
        'required' => $item->required,
        'options' => $options,
        'converter' => $type === 'delayed' ? $converter : $this->getConverter(),
      ));
    }

    return $items;
  }

  /**
   * @inheritdoc
   */
  public function getItem($id) {
    $items = $this->getItems();
    return isset($items[$id]) ? $items[$id] : FALSE;
  }

  /**
   * @inheritdoc
   */
  public function getSourceAttributes() {
    // @todo use a cache class.
    // @todo handle languages properly
    static $attributes;

    if (!$languageId = $this->getConverter()->getProfile()->getLanguage()) {
      return array();
    }

    if (!isset($attributes[$languageId])) {
      $attributes[$languageId] = array();

      foreach ($this->converter->getProfile()->getConfigurationTypes('attribute', $languageId) as $id => $attribute) {
        $attribute += unserialize($attribute['data']);
        unset($attribute['data']);

        if (isset($attribute['attribute_type_code'])) {
          $class = __NAMESPACE__ . '\\Source\\MapsSystem\\Attribute\\' . maps_suite_drupal2camelcase($attribute['attribute_type_code']);
          $attributes[$languageId]['attribute:' . $id] = class_exists($class) ? new $class($attribute) : new MS\Attribute\DefaultAttribute($attribute);
        }
      }
    }

    return $attributes[$languageId];
  }

  /**
   * @inheritdoc
   */
  public function getSourceAttribute($id) {
    $attributes = $this->getSourceAttributes();
    return isset($attributes[$id]) ? $attributes[$id] : new BrokenAttribute(array('id' => $id));
  }

  /**
   * Build the property handlers from property definitions.
   *
   * @param $properties
   *   The list of properties.
   *
   * @return array
   */
  protected function buildSourcePropertyHandlers(array $properties) {
    $handlers = array();

    foreach ($properties as $id => $property) {
      $property += array('class' => 'DefaultProperty', 'id' => $id);
      $class = __NAMESPACE__ . '\\Source\\MapsSystem\\Property\\' . $property['class'];

      if (class_exists($class) && @is_subclass_of($class, __NAMESPACE__ . '\\Source\\MapsSystem\\PropertyWrapper')) {
        $handlers['property:' . $id] = new $class($property);
      }
    }

    return $handlers;
  }

  /**
   * @inheritdoc
   */
  public function getSourceProperty($name) {
    $properties = $this->getSourceProperties();
    return isset($properties[$name]) ? $properties[$name] : new MS\Property\Broken();
  }

  /**
   * @inheritdoc
   */
  public function getTargetFields() {
    global $language;
    // @todo use a cache class.
    // @todo handle languages properly
    $fields = array();

    $info = entity_get_property_info($this->getConverter()->getEntityType()) + array('properties' => array());
    $properties = $info['properties'];

    if (isset($info['bundles'][$this->getConverter()->getBundle()]['properties'])) {
      $properties += $info['bundles'][$this->getConverter()->getBundle()]['properties'];
    }

    foreach ($properties as $name => $property) {
      if (isset($property['maps_import_handler'])) {
        $property['id'] = $name;
        $fields[$name] = new $property['maps_import_handler']($property);
      }
    }

    return $fields;
  }

  /**
   * @inheritdoc
   */
  public function getTargetField($name) {
    $fields = $this->getTargetFields();

    // @todo create the Broken class for Drupal fields and add an additional
    // use statement on the top of current file with an alias of BrokenField.
    return isset($fields[$name]) ? $fields[$name] : new Drupal\Field\Broken(array());
  }

  /**
   * @inheritdoc
   */
  public function getMappedEntities($uid) {
    $query = db_select(MappingInterface::DB_ENTITIES_TABLE, 'e')
      ->fields('e')
      ->condition('e.entity_type', $this->getConverter()->getEntityType())
      ->condition('e.bundle', $this->getConverter()->getBundle())
      ->condition('e.uid', $uid)
      ->condition(db_or()
        ->condition('e.uid_scope', array(0, 1))
        ->condition(db_and()
          ->condition('e.uid_scope', 2)
          ->condition('e.pid', $this->getConverter()->getProfile()->getPid())))
      ->orderBy('abs(e.pid - ' . $this->getConverter()->getProfile()->getPid() . ')', 'ASC')
      ->orderBy('abs(e.cid - ' . $this->getConverter()->getCid() . ')', 'ASC');

    // @todo this could override some other existing contents...
    $result = $query->execute()->fetchAllAssoc('id_language', \PDO::FETCH_ASSOC);
    return $result ? $result : array();
  }

  /**
   * @inheritdoc
   */
  public static function getEntityFromMapsId($type, $mapsId, $uid_scope = 1) {
  	$table = 'maps_import_' . $type . '_ids';

    $query = db_select(MappingInterface::DB_ENTITIES_TABLE, 'e');
    $query->join($table, 't', 'e.id = t.correspondence_id');
    $query
      ->fields('e')
      ->condition('e.uid_scope', $uid_scope)
      ->condition('t.maps_id', $mapsId);

    $result = $query->execute()->fetchAllAssoc('id_language', \PDO::FETCH_ASSOC);

    return !empty($result) ? $result : array();
  }

  /**
   * @inheritdoc
   */
  public static function getEntityIdFromMapsId($type, $mapsId, $uid_scope = 1) {
  	if ($result = self::getEntityFromMapsId($type, $mapsId, $uid_scope)) {
  		foreach ($result as &$row) {
        $row = $row['entity_id'];
  		}
    }

    return !empty($result) ? $result : array();
  }

  /**
   * @inheritdoc
   */
  public static function getCurrentEntity() {
    return self::$currentEntity;
  }

}
